SystemOrganization addCategory: #MaybeTests!

TestCase subclass: #JustTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MaybeTests'!

!JustTest methodsFor: 'testing - monad' stamp: 'fbs 6/29/2011 07:40'!
testBindWithJustReturnsJustWithFunctionApplied
	| j |
	j := (Just value: 2) >>= [:x | Just value: (x * 2)].
	self assert: j hasValue.
	self assert: j value = 4.! !

!JustTest methodsFor: 'testing - monad' stamp: 'fbs 6/29/2011 07:40'!
testBindWithNothingReturnsNothing
	| j |
	j := (Just value: 2) >>= [:x | Maybe nothing].
	self deny: j hasValue.! !

!JustTest methodsFor: 'testing - comparing' stamp: 'fbs 7/5/2011 17:57'!
testDifferentValueIsNotEqual
	self deny: (Just value: 1) = (Just value: 2).! !

!JustTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:33'!
testIfNothingIfNotNothingRunsJustBlock
	self assert: 2 == ((Maybe value: 42) ifNothing: [1] ifNotNothing: [2])! !

!JustTest methodsFor: 'testing - running' stamp: 'fbs 6/15/2011 17:52'!
testMaybeDefaultReturnsBlockValue
	| block default result thing |
	default := Object new.
	block := [:x | x * 2].
	thing := Maybe value: 2.
	result := thing maybe: block default: default.
	self assert: result = (block value: thing value).! !

!JustTest methodsFor: 'testing - comparing' stamp: 'fbs 7/5/2011 17:59'!
testNothingIsNotEqualToJust
	self deny: (Just value: 1) = Maybe nothing.
	self deny: (Just value: 1) = Nothing new.! !

!JustTest methodsFor: 'testing - printing' stamp: 'fbs 6/15/2011 17:41'!
testPrintStringLooksLikeHaskell
	self assert: (Just value: 2) printString = 'Just 2'.! !

!JustTest methodsFor: 'testing - comparing' stamp: 'fbs 7/5/2011 17:57'!
testSameObjectIsEqual
	| j |
	j := Just value: 1.
	self assert: j = j.! !

!JustTest methodsFor: 'testing - comparing' stamp: 'fbs 7/5/2011 17:56'!
testSameValueJustIsEqual
	self assert: (Just value: 1) = (Just value: 1).! !

TestCase subclass: #NothingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MaybeTests'!

!NothingTest methodsFor: 'testing - monad' stamp: 'fbs 6/29/2011 07:39'!
testBindReturnsNothing
	self deny: (Maybe nothing >>= [:x | Just value: x * 2]) hasValue.
	self deny: ((Just value: 1) >>= [:x | Maybe nothing]) hasValue! !

!NothingTest methodsFor: 'testing - monad' stamp: 'fbs 6/29/2011 07:39'!
testBindWithJustReturnsNothing
	self deny: (Maybe nothing >>= [:x | Just value: x * 2]) hasValue.! !

!NothingTest methodsFor: 'testing - monad' stamp: 'fbs 6/29/2011 07:40'!
testBindWithNothingReturnsNothing
	self deny: (Maybe nothing >>= [:x | Just value: x * 2]) hasValue! !

!NothingTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:34'!
testIfNothingIfNotNothingRunsNothingBlock
	self assert: 1 == (Maybe nothing ifNothing: [1] ifNotNothing: [2]).
	self assert: 1 == ((Nothing new) ifNothing: [1] ifNotNothing: [2]).! !

!NothingTest methodsFor: 'testing - applying' stamp: 'fbs 6/15/2011 17:55'!
testMaybeDefaultReturnsDefault
	| block default result |
	default := Object new.
	block := [:x | x * 2].
	result := Maybe nothing maybe: block default: default.
	self assert: result = default.! !

!NothingTest methodsFor: 'testing - printing' stamp: 'fbs 6/15/2011 17:40'!
testPrintStringLooksLikeHaskell
	self assert: Maybe nothing printString = 'Nothing'.! !

MonadTestCase subclass: #MaybeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MaybeTests'!

!MaybeTest methodsFor: 'data' stamp: 'fbs 7/6/2011 07:46'!
f
	^ [:x | Just value: (x + 1)].! !

!MaybeTest methodsFor: 'data' stamp: 'fbs 7/6/2011 07:47'!
g
	^ [:x | Just value: (x * 2)].! !

!MaybeTest methodsFor: 'initialize-release' stamp: 'fbs 7/6/2011 07:47'!
initialize
	monad := Maybe.! !

!MaybeTest methodsFor: 'data' stamp: 'fbs 7/6/2011 07:48'!
monad
	^ Maybe.! !

!MaybeTest methodsFor: 'data' stamp: 'fbs 7/6/2011 07:40'!
sampleBlock
	^ [:x | Just value: (x + 1)].! !

!MaybeTest methodsFor: 'data' stamp: 'fbs 7/6/2011 07:39'!
sampleValues
	^ #(1 '2' #(3)).! !

!MaybeTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:27'!
testNothingHasNoValue
	self deny: Maybe nothing hasValue.
	
	self should: [Maybe nothing value] raise: Error.! !

!MaybeTest methodsFor: 'testing - instance creation' stamp: 'fbs 6/15/2011 17:26'!
testNothingIsIdempotent
	self assert: Maybe nothing == Maybe nothing.! !

!MaybeTest methodsFor: 'testing - monad' stamp: 'fbs 6/28/2011 22:15'!
testReturnAddsObjectToMonad
	| m |
	m := Maybe return: 1.
	self assert: m hasValue.! !

!MaybeTest methodsFor: 'testing - instance creation' stamp: 'fbs 6/15/2011 17:24'!
testValueReturnsaJust
	| r |
	r := Maybe value: 1.
	self assert: r hasValue.! !
